import "package:chat_repository_interface/chat_repository_interface.dart";
import "package:flutter/material.dart";
import "package:flutter_chat/src/config/chat_translations.dart";
import "package:flutter_chat/src/config/screen_types.dart";
import "package:flutter_chat/src/screens/chat_detail/widgets/default_loader.dart";
import "package:flutter_chat/src/screens/chat_detail/widgets/default_message_builder.dart";

/// The chat builders
class ChatBuilders {
  /// The chat builders constructor
  const ChatBuilders({
    this.baseScreenBuilder,
    this.messageInputBuilder,
    this.chatRowContainerBuilder,
    this.groupAvatarBuilder,
    this.imagePickerContainerBuilder,
    this.userAvatarBuilder,
    this.deleteChatDialogBuilder,
    this.newChatButtonBuilder,
    this.noUsersPlaceholderBuilder,
    this.chatTitleBuilder,
    this.chatMessageBuilder = DefaultChatMessageBuilder.builder,
    this.usernameBuilder,
    this.loadingWidgetBuilder = DefaultChatLoadingOverlay.builder,
    this.loadingChatMessageBuilder = DefaultChatMessageLoader.builder,
  });

  /// The base screen builder
  /// This builder is used to build the base screen for the chat
  /// You can switch on the [screenType] to build different screens
  /// ```dart
  ///   baseScreenBuilder: (context, screenType, appBar, body) {
  ///     switch (screenType) {
  ///       case ScreenType.chatScreen:
  ///         return Scaffold(
  ///           appBar: appBar,
  ///           body: body,
  ///         );
  ///       case ScreenType.chatDetailScreen:
  ///       // And so on....
  /// ```
  final BaseScreenBuilder? baseScreenBuilder;

  /// The message input builder
  final TextInputBuilder? messageInputBuilder;

  /// The chat row container builder
  final ContainerBuilder? chatRowContainerBuilder;

  /// The group avatar builder
  final GroupAvatarBuilder? groupAvatarBuilder;

  /// The user avatar builder
  final UserAvatarBuilder? userAvatarBuilder;

  /// The delete chat dialog builder
  final Future<bool?> Function(BuildContext, ChatModel)?
      deleteChatDialogBuilder;

  /// The new chat button builder
  final ButtonBuilder? newChatButtonBuilder;

  /// The no users placeholder builder
  final NoUsersPlaceholderBuilder? noUsersPlaceholderBuilder;

  /// The chat title builder
  final Widget Function(String chatTitle)? chatTitleBuilder;

  /// The chat message builder
  final ChatMessageBuilder chatMessageBuilder;

  /// The username builder
  final Widget Function(String userFullName)? usernameBuilder;

  /// The image picker container builder
  final ImagePickerContainerBuilder? imagePickerContainerBuilder;

  /// The loading widget builder
  /// This is used to build the loading widget that is displayed on the chat
  /// screen when loading the chat
  final WidgetBuilder loadingWidgetBuilder;

  /// The loading widget builder for chat messages
  /// This is displayed in the list of chat messages when loading more messages
  /// can be above and below the list
  final WidgetBuilder loadingChatMessageBuilder;
}

/// The button builder
typedef ButtonBuilder = Widget Function(
  BuildContext context,
  VoidCallback onPressed,
  ChatTranslations translations,
);

/// The image picker container builder
typedef ImagePickerContainerBuilder = Widget Function(
  BuildContext context,
  VoidCallback onClose,
  ChatTranslations translations,
);

/// The text input builder
typedef TextInputBuilder = Widget Function(
  BuildContext context,
  TextEditingController textEditingController,
  Widget suffixIcon,
  ChatTranslations translations,
  VoidCallback onSubmit,
);

/// The base screen builder
/// [title] is the title of the screen and can be null while loading
typedef BaseScreenBuilder = Widget Function(
  BuildContext context,
  ScreenType screenType,
  PreferredSizeWidget appBar,
  String? title,
  Widget body,
);

/// The container builder
typedef ContainerBuilder = Widget Function(
  BuildContext context,
  Widget child,
);

/// The chat message builder
/// This builder is used to override the default chat message widget
/// If null is returned, the default chat message widget will be used so you can
/// override for specific cases
/// [previousMessage] is the previous message in the chat
/// [sender] is the sender of the message and null if no user sent the message
typedef ChatMessageBuilder = Widget? Function(
  BuildContext context,
  MessageModel message,
  MessageModel? previousMessage,
  UserModel? sender,
  Function(UserModel sender) onPressSender,
);

/// The group avatar builder
typedef GroupAvatarBuilder = Widget Function(
  BuildContext context,
  String groupName,
  String? imageUrl,
  double size,
);

/// The user avatar builder
typedef UserAvatarBuilder = Widget Function(
  BuildContext context,
  UserModel user,
  double size,
);

/// The no users placeholder builder
typedef NoUsersPlaceholderBuilder = Widget Function(
  BuildContext context,
  ChatTranslations translations,
);
